@@ -16,10 +16,21 @@
 """Container for generic objects."""
 
 
-from uuid import uuid4
+import signal
+
+from functools import wraps
 from threading import Condition, Lock
 
 
+# Pool states
+STOPPED = 0
+RUNNING = 1
+CLOSED = 2
+CREATED = 3
+EXPIRED = 4
+ERROR = 5
+
+
 # --------------------------------------------------------------------------- #
 #                                 Exceptions                                  #
 # --------------------------------------------------------------------------- #
@@ -36,8 +47,58 @@ class TaskCancelled(PebbleError):
 class TimeoutError(PebbleError):
     """Raised when a timeout expires."""
     def __init__(self, msg, value=0):
-        self.msg = msg
-        self.value = value
+        super(TimeoutError, self).__init__(msg)
+        self.timeout = value
+
+
+class ProcessExpired(PebbleError):
+    """Raised when process dies unexpectedly."""
+    def __init__(self, msg, code=0):
+        super(ProcessExpired, self).__init__(msg)
+        self.exitcode = code
+
+
+# --------------------------------------------------------------------------- #
+#                                 Decorators                                  #
+# --------------------------------------------------------------------------- #
+def synchronized(lock):
+    """Locks the execution of decorated function on given *lock*.
+
+    Works with both threading and multiprocessing Lock.
+
+    """
+    def wrap(function):
+        @wraps(function)
+        def wrapper(*args, **kwargs):
+            with lock:
+                return function(*args, **kwargs)
+
+        return wrapper
+
+    return wrap
+
+
+def sighandler(signals):
+    """Sets the decorated function as signal handler of given *signals*.
+
+    *signals* can be either a single signal or a list/tuple
+    of multiple ones.
+
+    """
+    def wrap(function):
+        if isinstance(signals, (list, tuple)):
+            for signum in signals:
+                signal.signal(signum, function)
+        else:
+            signal.signal(signals, function)
+
+        @wraps(function)
+        def wrapper(*args, **kwargs):
+            return function(*args, **kwargs)
+
+        return wrapper
+
+    return wrap
 
 
 # --------------------------------------------------------------------------- #
@@ -47,7 +108,7 @@ class Task(object):
     """Handler to the ongoing task."""
     def __init__(self, task_nr, function=None, args=None, kwargs=None,
                  callback=None, timeout=0, identifier=None):
-        self.id = identifier is not None and identifier or uuid4()
+        self.id = identifier
         self.timeout = timeout
         self._function = function
         self._args = args
